/*$Id: d_cccs.cc,v 13.40 1999/05/19 08:40:21 al Exp $ -*- C++ -*-
 * functions for cccs
 * It is really voltage controlled, taking into account the sense element.
 * Then adjust the gain to account for the sense element.
 */
#include "d_cccs.h"
/*--------------------------------------------------------------------------*/
//	void	DEV_CCCS::expand();
// 	bool	DEV_CCCS::dotr();
// 	void	DEV_CCCS::doac();
/*--------------------------------------------------------------------------*/
CARD& DEV_CCCS::expand()
{
  if (*inputlabel){
    input = dynamic_cast<ELEMENT*>(find_in_scope(inputlabel));
  }else{
    // input already set, an internal element.  example: mutual L.
  }
  if (!input){
    untested();
    error(bERROR,"%s: %s cannot be used as current probe\n",
	  printlabel(),inputlabel);
  }
  assert(input);
  if (input->is2port()){
    n[IN1] = input->n[IN1];
    n[IN2] = input->n[IN2];
  }else{
    assert(input->is1port()  ||  input->issource());
    n[IN1] = input->n[OUT1];
    n[IN2] = input->n[OUT2];
  }
  n[IN1].e = n[IN2].e = INVALIDNODE;
  assert(loss == 0.);
  m0.f1 = y0.f1 = value();
  y0.f0 = LINEAR;
  m0.c0 = 0.;
  acg = ev = value();
  set_converged();
  assert(!constant()); /* because of incomplete analysis */
  return *this;
}
/*--------------------------------------------------------------------------*/
bool DEV_CCCS::dotr()
{
  assert(!isbypassed());
  m0.x = n[IN1].v0() - n[IN2].v0();
  y0.x = input->tr_amps();
  if (has_tr_eval()){
    tr_eval();
    m0.c0 = y0.f0 - y0.x * y0.f1;
  }else{
    assert(converged());
    y0.f1 = value();
    y0.f0 = y0.x * y0.f1;
    m0.c0 = 0.;
  }
  m0.c0 += y0.f1 * input->m0.c0;
  m0.f1  = y0.f1 * (input->loss + input->m0.f1);
  store_values();
  SIM::loadq.push(this);
  return converged();
}
/*--------------------------------------------------------------------------*/
CARD& DEV_CCCS::doac()
{
  if (!input->evaluated()){	/* patch for forward reference */
    input->doac();		/* make sure sense elt is evaluated first */
  }
  if (has_ac_eval()){
    acbias = n[IN1].vdc() - n[IN2].vdc();
    ac_eval();
  }else{
    ev  = y0.f1;
  }
  if (input->issource()){	/* if the sense elt is a fixed source.. */
    acg = ev * input->acg;	/* then part of this one can be modeled */
    acload_source();		/* as a fixed source. ...		*/
    acg = ev * input->loss;	/* so load it in 2 pieces		*/
    acload_active();
  }else{
    acg = ev * input->acg;
    acload_active();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
