/*$Id: d_coil.cc,v 13.34 1999/05/03 08:18:39 al Exp $ -*- C++ -*-
 * inductors
 * two levels: linear (lin) and nonlinear (nl) (not really)
 * x = amps, y.f0 = flux, ev = y.f1 = henrys
 */
#include "d_cccs.h"
#include "ap.h"
#include "d_coil.h"
/*--------------------------------------------------------------------------*/
//		DEV_MUTUAL_L::DEV_MUTUAL_L();
//		DEV_MUTUAL_L::DEV_MUTUAL_L(const DEV_MUTUAL_L& p);
//	void	DEV_MUTUAL_L::parse(CS&);
//	void	DEV_MUTUAL_L::print(int where, int detail)const;
//	void	DEV_MUTUAL_L::expand();

// 	bool	DEV_INDUCTANCE::dotr();
// 	void	DEV_INDUCTANCE::trload();
// 	void	DEV_INDUCTANCE::trunload();
// 	void	DEV_INDUCTANCE::doac();
//	double	DEV_INDUCTANCE::tr_review();
//	void	DEV_INDUCTANCE::integrate();
/*--------------------------------------------------------------------------*/
DEV_MUTUAL_L::DEV_MUTUAL_L()
{
  inputlabel[0] = outputlabel[0] = '\0';
  output = input = NULL;
}
/*--------------------------------------------------------------------------*/
DEV_MUTUAL_L::DEV_MUTUAL_L(const DEV_MUTUAL_L& p):COMPONENT(p)
{
  untested();
  strcpy(outputlabel, p.outputlabel);
  strcpy(inputlabel,  p.inputlabel);
  output = input = NULL;
}
/*--------------------------------------------------------------------------*/
CARD& DEV_MUTUAL_L::parse(CS& cmd)
{
  parselabel(cmd);
  cmd.ctostr(outputlabel, LABELEN, TOKENTERM);
  outputlabel[0] = toupper(outputlabel[0]);
  cmd.ctostr(inputlabel, LABELEN, TOKENTERM);
  inputlabel[0] = toupper(inputlabel[0]);
  setvalue(cmd.ctof());
  return *this;
}
/*--------------------------------------------------------------------------*/
const CARD& DEV_MUTUAL_L::print(OMSTREAM where, int)const
{
  printlabel(where);
  if (output){
    where << ' ';
    output->printlabel(where);
  }else{
    where << "  " << outputlabel;
  }
  if (input){
    where << ' ';
    input->printlabel(where);
  }else{
    where << "  " << inputlabel;
  }
  where.setfloatwidth(7) << ' ' << value() << '\n';
  return *this;
}
/*--------------------------------------------------------------------------*/
// replace both primary and secondary (both simple L's)
// with the CCCS - L equivalent
// only works for 2.
CARD& DEV_MUTUAL_L::expand()
{
  output = find_in_scope(outputlabel);
  input  = find_in_scope(inputlabel);
  output->subckt().destroy();
  input->subckt().destroy();

  double l1 = output->value();
  double l2 = input->value();
  double lm  = value() * sqrt(l1 * l2);
  double det = l1 * l2 - lm * lm;
  
  DEV_INDUCTANCE* pri = dynamic_cast<DEV_INDUCTANCE*>(output->clone());
  if (!pri){
    untested();
    error(bERROR,"%s: %s is not an inductor\n",printlabel(),outputlabel);
  }
  pri->set_owner(output);
  pri->setvalue(det / l2);
  pri->ev = pri->y0.f1 = value();
  pri->y0.f0 = LINEAR;
  output->subckt().push_back(pri);

  DEV_INDUCTANCE* sec = dynamic_cast<DEV_INDUCTANCE*>(input->clone());
  if (!sec){
    untested();
    error(bERROR,"%s: %s is not an inductor\n",printlabel(),inputlabel);
  }
  sec->set_owner(input);
  sec->setvalue(det / l1);
  sec->ev = sec->y0.f1 = value();
  sec->y0.f0 = LINEAR;
  input->subckt().push_back(sec);

  DEV_CCCS* sub = new DEV_CCCS;
  assert(sub);
  sub->set_owner(output);
  sub->input   = sec;
  sprintf(sub->label, "F%s", &sub->input->label[1]);
  sub->n[OUT1] = output->n[OUT1];
  sub->n[OUT2] = output->n[OUT2];
  sub->setvalue(-lm / l1);
  sub->ev = sub->y0.f1 = sub->value();
  sub->y0.f0 = LINEAR;
  output->subckt().push_back(sub).expand();
  
  sub = new DEV_CCCS;
  assert(sub);
  sub->set_owner(input);
  sub->input   = pri;
  sprintf(sub->label, "F%s", &sub->input->label[1]);
  sub->n[OUT1] = input->n[OUT1];
  sub->n[OUT2] = input->n[OUT2];
  sub->setvalue(-lm / l2);
  sub->ev = sub->y0.f1 = sub->value();
  sub->y0.f0 = LINEAR;
  input->subckt().push_back(sub).expand();

  assert(!constant()); /* because of integration */
  return *this;
}
/*--------------------------------------------------------------------------*/
bool DEV_INDUCTANCE::dotr()
{
  assert(!isbypassed());
  if (subckt().exists()){
    set_converged(subckt().dotr());
  }else{
    initial_voltage = initial_current = initial_condition = NOT_INPUT;
    advance();
    if (has_tr_eval()){
      m0.x = volts_limited(n[OUT1],n[OUT2]);
      y0.x = m0.c0 + m0.f1 * m0.x;
      tr_eval();
    }else{
      m0.x = n[OUT1].v0() - n[OUT2].v0();
      y0.x = m0.c0 + m0.f1 * m0.x;
      y0.f1 = value();
      y0.f0 = y0.x * y0.f1;
      assert(converged());
    }
    if (y0.f1 == 0.){
      error(bPICKY, "%s: short circuit\n", printlabel());
    }
    store_values();
    SIM::loadq.push(this);
    integrate();
  }
  return converged();
}
/*--------------------------------------------------------------------------*/
CARD& DEV_INDUCTANCE::trload()
{
  if (subckt().exists()){
    subckt().trload();
  }else{
    trload_passive();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
CARD& DEV_INDUCTANCE::trunload()
{
  if (subckt().exists()){
    untested();
    subckt().trunload();
  }else{
    untested();
    trunload_passive();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
CARD& DEV_INDUCTANCE::doac()
{
  if (subckt().exists()){
    subckt().doac();
  }else{
    if (has_ac_eval()){
      double dcvolts = n[OUT1].vdc() - n[OUT2].vdc();
      acbias = m0.c0 + m0.f1*dcvolts;
      ac_eval();
    }else if (has_tr_eval()){
      untested();
      ev = y0.f1;
    }else{
      assert(ev == value());
    }
    if (SIM::jomega == 0.){
      untested();
      acg = 1. / OPT::shortckt;
      error(bPICKY, "%s: short circuit\n", printlabel());
    }else{
      acg = 1. / (ev * SIM::jomega);
    }
    acload_passive();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
double DEV_INDUCTANCE::tr_review()
{
  if (subckt().exists()){
    return timef = subckt().tr_review();
  }else{
    return review(m0.x, mt1.x);    
  }
}
/*--------------------------------------------------------------------------*/
void DEV_INDUCTANCE::integrate()
{
  if (SIM::mode == sDC  ||  SIM::mode == sOP  ||  y0.f1 == 0.){
    m0.f1 = 1./OPT::shortckt;
    m0.c0 = 0.;
  }else if (SIM::phase == pINIT_DC){
    if (time0 == 0.){
      if (!SIM::uic){
	m0.f1 = 1./OPT::shortckt;
	m0.c0 = 0.;
      }else if (initial_current != NOT_INPUT){
	untested();
	m0.f1 = 0.;
	m0.c0 = -initial_current;
      }else if (initial_voltage != NOT_INPUT){
	untested();
	m0.f1 = 1./OPT::shortckt;
	m0.c0 = -initial_voltage * m0.f1;
      }else if (initial_condition != NOT_INPUT){
	untested();
	m0.f1 = 0.;
	m0.c0 = -initial_condition;
      }else{
	untested();
	m0.f1 = 1./OPT::shortckt;
	m0.c0 = 0.;
      }
    }else{
      untested();
      /* leave it alone to restore */
    }
  }else{
    double dt = time0 - time1;
    switch (method_a){
    case mGEAR:
    case mTRAPGEAR:
    case mTRAPEULER:
    case mEULER:
      untested();
      m0.f1 = dt / y0.f1;
      m0.c0 = mt1.c0 + mt1.f1 * mt1.x; /* oldi */
      break;
    case mTRAP:
      /* oldi = mt1.c0 + mt1.f1 * mt1.x; */
      m0.f1 = dt/(2*y0.f1);
      m0.c0 = mt1.c0 + (mt1.f1 + m0.f1) * mt1.x;
      /* oldi + f1*oldv, with combined terms */
      break;
    }
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
