/*$Id: d_switch.h,v 13.32 1999/04/25 07:12:45 al Exp $ -*- C++ -*-
 * data structures for voltage (and current) controlled switch
 */
#ifndef D_SWITCH_H
#define D_SWITCH_H
#include "e_model.h"
#include "e_elemnt.h"
/*--------------------------------------------------------------------------*/
class MODEL_SWITCH : public MODEL_CARD {
friend class SWITCH_BASE;
private:
  explicit	MODEL_SWITCH(const MODEL_SWITCH& p) 
				:MODEL_CARD(p) {unreachable();}
public:
  explicit	MODEL_SWITCH();
private: // overloaded virtual
  CARD*		clone()const	{untested(); return new MODEL_SWITCH(*this);}
  CARD&		parse(CS&);
  const CARD&	print(OMSTREAM,int)const;
private:
  double    vt;		/* threshold voltage */
  double    vh;		/* hysteresis voltage */
  double    ron;	/* on resistance */
  double    roff;	/* off resistance */
  enum control_t {VOLTAGE, CURRENT};
  control_t type;	/* current or voltage controlled */
};
/*--------------------------------------------------------------------------*/
class SWITCH_COMMON : public COMPONENT_COMMON {
public:
  explicit SWITCH_COMMON(int c=0)		:COMPONENT_COMMON(c) {}
  explicit SWITCH_COMMON(const SWITCH_COMMON& p):COMPONENT_COMMON(p) {}
  const char* name()const			{untested(); return "switch";}
};
/*--------------------------------------------------------------------------*/
class SWITCH_BASE : public ELEMENT {
protected:
  explicit	SWITCH_BASE();
  explicit	SWITCH_BASE(const SWITCH_BASE& p);
  SWITCH_BASE&	parse_sb(CS&,int);
  SWITCH_BASE&	expand_sb();
private: // overloaded virtual
  const CARD&	print(OMSTREAM,int)const;
  bool		dotr();
  CARD&		trload()		{return trload_passive();}
  CARD&		trunload()		{untested();return trunload_passive();}
  CARD&		doac();
  double	tr_involts()const	{untested(); return tr_outvolts();}
  COMPLEX	ac_involts()const	{untested(); return ac_outvolts();}
  int		numnodes()const		{return 4;}
  bool		is1port()const		{untested(); return true;}
protected:
  char		inputlabel[LABELEN+1];/* this is here instead of in Cswitch */
  ELEMENT*	input;		      /* due to bad design and lazyness */
  enum	{IPRINTNODES = 2, VPRINTNODES = 4};
private:
  enum state_t {UNKNOWN, ON, OFF};
  state_t	ic;		/* initial state, belongs in common */
  state_t	current_state;	/* state 1 iter ago (may be bad) */
  state_t	previous_state;	/* state 1 time or step ago (known good) */
};
/*--------------------------------------------------------------------------*/
class DEV_VSWITCH : public SWITCH_BASE {
private:
  explicit  DEV_VSWITCH(const DEV_VSWITCH& p):SWITCH_BASE(p){untested();}
public:
  explicit  DEV_VSWITCH()	{}
private: // overloaded virtual
  CARD*	    clone()const	{untested();return new DEV_VSWITCH(*this);}
  CARD&	    parse(CS& cmd)	{return parse_sb(cmd,4);}
  CARD&	    expand()		{return expand_sb();}
  const char* devtype()const	{return "vswitch";}
};
/*--------------------------------------------------------------------------*/
class DEV_CSWITCH : public SWITCH_BASE {
private:
  explicit  DEV_CSWITCH(const DEV_CSWITCH& p):SWITCH_BASE(p){untested();}
public:
  explicit  DEV_CSWITCH()	{}
private: // overloaded virtual
  CARD*	    clone()const	{untested();return new DEV_CSWITCH(*this);}
  CARD&	    parse(CS& cmd)	{return parse_sb(cmd,2);}
  CARD&	    expand();
  const char* devtype()const	{return "iswitch";}
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
