/* $Id: s__init.cc,v 13.36 1999/05/07 08:37:41 al Exp $
 * initialization (allocation, node mapping, etc)
 */
#include "e_card.h"
#include "u_status.h"
#include "l_jmpbuf.h"
#include "s__.h"
/*--------------------------------------------------------------------------*/
//	 void	SIM::command_base(CS&);
//static void	SIM::init()
//	 void	SIM::reset_timers();
//static void	SIM::count_nodes();
//static void	SIM::alloc_hold_vectors();
//static void	SIM::determine_matrix_structure(const CARD *stop);
//	 void	SIM::alloc_vectors();
//static void	SIM::unalloc_vectors();
//static void	SIM::uninit();
/*--------------------------------------------------------------------------*/
extern JMP_BUF env;
extern run_mode_t run_mode;
extern NODE* nstat;
/*--------------------------------------------------------------------------*/
void SIM::command_base(CS& cmd)
{
  reset_timers();
  init();
  alloc_vectors();
  aa.allocate().dezero(OPT::gmin).setminpivot(OPT::pivtol);
  lu.allocate().dezero(OPT::gmin).setminpivot(OPT::pivtol);
  //loadq.reserve(CARD::devicecount());
  JMP_BUF stash = env;
  if (!setjmp(env.p)){ // try
    setup(cmd);
    STATUS::set_up.stop();
    if (run_mode == rEXECUTE)
      sweep();
    lu.unallocate();
    aa.unallocate();
    unalloc_vectors();
    env = stash;
    finish();
  }else{ // catch
    untested();
    env = stash;
    finish();
    error(bERROR, "");
  }
  STATUS::total.stop();
}
/*--------------------------------------------------------------------------*/
/* init: allocate, set up, etc ... for any type of simulation
 * also called by status and probe for access to internals and subckts
 */
/*static*/ void SIM::init()
{
  if (!nstat){
    uninit();
    count_nodes();
    CARD_LIST::card_list.expand();
    map_nodes();
    alloc_hold_vectors();
    aa.reinit(STATUS::total_nodes);
    lu.reinit(STATUS::total_nodes);
    acx.reinit(STATUS::total_nodes);
    determine_matrix_structure(CARD_LIST::card_list);
  }
}
/*--------------------------------------------------------------------------*/
void SIM::reset_timers()
{
  STATUS::evaluate.reset();
  STATUS::load.reset();
  STATUS::lud.reset();
  STATUS::back.reset();
  STATUS::review.reset();
  STATUS::output.reset();
  STATUS::aux1.reset();
  STATUS::aux2.reset();
  STATUS::aux3.reset();
  STATUS::set_up.reset().start();
  STATUS::total.reset().start();
  STATUS::iter[mode] = 0;
  STATUS::iter[iPRINTSTEP] = 0;
}
/*--------------------------------------------------------------------------*/
/* count_nodes: count nodes in main ckt (not subckts)
 * update the variables "STATUS::total_nodes" and "STATUS::user_nodes"
 * zeros "STATUS::subckt_nodes" and "STATUS::model_nodes"
 */
/*static*/ void SIM::count_nodes()
{
  STATUS::user_nodes = 0;
  for (CARD_LIST::const_iterator
	 ci=CARD_LIST::card_list.begin();ci!=CARD_LIST::card_list.end();++ci){
    CARD* brh = *ci;
    if (brh->isdevice()){
      for (int ii = 0;  brh->n[ii].e != INVALIDNODE;  ii++){
	if (brh->n[ii].e > STATUS::user_nodes){
	  STATUS::user_nodes = brh->n[ii].e;
	}
      }
    }
  }
  STATUS::total_nodes = STATUS::user_nodes;
  STATUS::subckt_nodes = STATUS::model_nodes = 0;
}
/*--------------------------------------------------------------------------*/
/* alloc_hold_vectors:
 * allocate space to hold data between commands.
 * for restart, convergence assistance, bias for AC, post-processing, etc.
 * must be done BEFORE deciding what array elements to allocate,
 * but after mapping
 * if they already exist, leave them alone to save data
 */
void SIM::alloc_hold_vectors()
{
  if (!nstat){
    nstat = new NODE[STATUS::total_nodes+1];
    assert(nstat);
    for (int ii=0;  ii <= STATUS::total_nodes;  ++ii){
      //nstat[NODE::to_internal(ii)].SetNumber(ii);
      nstat[nm[ii]].set_number(ii);
    }
  }else{
    untested();
  }
  if (!vdc){
    vdc = new double[STATUS::total_nodes+1];
    assert(vdc);
    for (int ii=0;  ii <= STATUS::total_nodes;  ++ii){
      vdc[ii] = 0.;
    }
  }else{
    untested();
  }
}
/*--------------------------------------------------------------------------*/
/* determine_matrix_structure: 
 * scan the list and tell the matrix which entries to allocate
 * recursive for subckts
 */
void SIM::determine_matrix_structure(const CARD_LIST& cl)
{
  for (CARD_LIST::const_iterator ci = cl.begin();  ci != cl.end();  ++ci){
    const CARD* brh = *ci;
    if (brh->isdevice()){
      for (int ii = 0;  brh->n[ii].m != INVALIDNODE;  ++ii){
	if (brh->n[ii].m != 0){
	  for (int jj = 0;  jj < ii ;  jj++){
	    aa.iwant(brh->n[ii].m,brh->n[jj].m);
	    lu.iwant(brh->n[ii].m,brh->n[jj].m);
	    acx.iwant(brh->n[ii].m,brh->n[jj].m);
	  }
	  nstat[brh->n[ii].m].set_needs_analog();
	}
      }
      if (brh->subckt().exists()){
	determine_matrix_structure(brh->subckt());
      }
    }
  }
}
/*--------------------------------------------------------------------------*/
/* alloc_vectors:
 * allocate space for the right-side vector
 * (initially current sources, on solution becomes voltages) and copies
 * used for one-time-ago and convergence checking
 * these are new with every run and are discarded after the run.
 */
void SIM::alloc_vectors()
{
  unalloc_vectors();	/* usually unnecessary. */
  assert(!ac);		/* may be needed after exception */
  assert(!i);
  assert(!v0);
  assert(!vi1);
  assert(!vt1);
  assert(!vt2);
  assert(!fw);
  ac = new COMPLEX[STATUS::total_nodes+1];
  i   = new double[STATUS::total_nodes+1];
  v0  = new double[STATUS::total_nodes+1];
  vi1 = new double[STATUS::total_nodes+1];
  vt1 = new double[STATUS::total_nodes+1];
  vt2 = new double[STATUS::total_nodes+1];
  fw  = new double[STATUS::total_nodes+1];
  assert(ac);
  assert(i);
  assert(v0);
  assert(vi1);
  assert(vt1);
  assert(vt2);
  assert(fw);
  for (int ii=0;  ii <= STATUS::total_nodes;  ++ii){
    ac[ii] = 0.;
    i[ii] = 0.;
    v0[ii] = 0.;
    vi1[ii] = 0.;
    vt1[ii] = 0.;
    vt2[ii] = 0.;
    fw[ii] = 0.;
  }
}
/*--------------------------------------------------------------------------*/
/*static*/ void SIM::unalloc_vectors()
{
  delete [] i;
  i = NULL;
  delete [] v0;
  v0 = NULL;
  delete [] vi1;
  vi1 = NULL;
  delete [] vt1;
  vt1 = NULL;
  delete [] vt2;
  vt2 = NULL;
  delete [] fw;
  fw = NULL;
  delete [] ac;
  ac = NULL;
}
/*--------------------------------------------------------------------------*/
/* uninit: undo all the allocation associated with any simulation
 * called when the circuit changes after a run, so it needs a restart
 * may be called multiple times without damage to make sure it is clean
 */
/*static*/ void SIM::uninit()
{
#if !defined(SGI_CC)	/* for some reason this generates bad code on 	    */
  delete [] nstat;	/* SGI Indy with the native compiler (NCC or DCC)   */
#endif			/* The linker reports something like "bad relocation*/
  nstat = NULL;		/* entry".  This workaround causes a memory leak,   */
  aa.reinit(0);		/* but at least you can compile it.		    */
  lu.reinit(0);		/* g++ (2.6.3) does not have the problem.	    */
  acx.reinit(0);
  delete [] nm;
  nm = NULL;
  delete [] vdc;
  vdc = NULL;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
