/*$Id: u_xprobe.h,v 13.28 1999/04/11 08:25:27 al Exp $ -*- C++ -*-
 * extended probe data
 * used for AC analysis
 */
#ifndef U_XPROBE_H
#define U_XPROBE_H
#include "constant.h"
#include "md.h"
/*--------------------------------------------------------------------------*/
typedef enum {mtNONE, mtMAG, mtPHASE, mtREAL, mtIMAG} mod_t;
/*--------------------------------------------------------------------------*/
class XPROBE{
private:
  COMPLEX value;
  mod_t   modifier; // default
  double  dbscale;  // 20 for voltage, 10 for power, etc.
public:  
  explicit XPROBE():
    value(COMPLEX(NOT_VALID, NOT_VALID)),
    modifier(mtNONE),
    dbscale(20.) {}
  explicit XPROBE(COMPLEX v,mod_t m=mtMAG,double d=20.):
    value(v),
    modifier(m),
    dbscale(d) {}
  explicit XPROBE(double v,mod_t m=mtREAL,double d=20.):
    value(v),
    modifier(m),
    dbscale(d) {untested();}
  bool OK()const {return modifier != mtNONE;}
  double operator()(mod_t m=mtNONE, bool db = false)const{
    if (OK()){
      if (m == mtNONE) {
	m = modifier;
      }
      double rv = NOT_VALID;
      switch (m){
      case mtNONE:  unreachable();		break;
      case mtMAG:   rv = std::abs(value);	break;
      case mtPHASE: rv = std::arg(value)*RTOD;	break;
      case mtREAL:  rv = std::real(value);	break;
      case mtIMAG:  rv = std::imag(value);	break;
      }
      return (db) ? dbscale * log10(max(rv,VOLTMIN)) : rv;
    }else{
      return NOT_VALID;
    }
  }
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
